#ifndef __LOCALIZATION_FILE_IO_H
#define __LOCALIZATION_FILE_IO_H

#include <iostream>
#include "stdio.h"
#include "global_vars.h"
using namespace std;

//These functions handle writing and reading to files.


//******************************************************//
//***Reads in trained txt file to create classifiers****//
//******************************************************//
// the file "target_classifiers.txt" must be in the root directory.
// this is computed by the other VC++ solution (modified by M. Hwang to fix bug)
classifier* LoadTargetData()
{
	ifstream fin("target_classifiers_augmented.txt");
	if (!fin.good()){
		cout<<"error opening 'target_classifiers_augmented.txt'"<<endl;
		while(1){}
	}
	string line;
	classifier* tail;
	classifier* current = new classifier;
	current->nxt = NULL;
	current->ID = -1;
	current->numDots = -1;
	current->angleoffset = -1;
	current->centermag = -1;     //magnitude, the distance from the center of mass of the dots to the center of the LED board, added 8/10
	current->centeroffset = -1;     //offset, this is the angle from the center of mass of the dots to the nearest used point, added 8/10

	classifier* first = current;
	while(!fin.eof())
	{
		//read one line of data from txt file
		fin>>line;

		string data;
		current->num_idd = 0;
		//process data, looking for commas
		while(line.find(",")!=string::npos)
		{
			data = line.substr(0, line.find(","));
			line = line.substr(line.find(",") + 1);
			
			if (current->ID == -1)
			{
				current->ID = atof(data.c_str());
			}
			else if (current->numDots == -1)
			{
				current->numDots= atof(data.c_str());
			}
			else if (current->angleoffset == -1)
			{
				current->angleoffset = atof(data.c_str());
			}
			else if (current->centermag == -1) //new 8/10
			{
				current->centermag = atof(data.c_str());
			}
			else if (current->centeroffset == -1) //new 8/10
			{
				current->centeroffset = atof(data.c_str());
			}
			else
			{
				current->idd[current->num_idd] = atof(data.c_str());
				current->num_idd++;
			}
		}
		//create a new classifier and initialize it
		classifier* new_classifier = new classifier;
		new_classifier->nxt = NULL;
		new_classifier->ID = -1;
		new_classifier->numDots = -1;
		new_classifier->angleoffset = -1;
		new_classifier->centermag = -1; //new 8/10
		new_classifier->centeroffset = -1; //new 8/10
		current->nxt = new_classifier;
		tail = current;
		current = current->nxt;
	}

	delete current;
	tail->nxt = NULL;
	fin.close();
	return first;
}


/* writes dot locations to file ""Quadrant%i.txt".*/
int WriteDotLocations( target *calib_pattern, int quadrant_i){
	char namestring[50];
	sprintf(namestring,"Quadrant%i.txt",quadrant_i);
	ofstream fout(namestring,ios_base::trunc);
	if (fout.fail()){
		cout<<"Failed to write "<<namestring<<".  Press Enter to continue..."<<endl;
		cin.get();
		return 0;
	}
	else{
		cout<<endl<<"Writing dot locations to "<<namestring<<endl<<endl;
		for (int i=0; i<9; i++){
			fout<<calib_pattern->dots[i].ix<<" "<<calib_pattern->dots[i].iy<<endl;
		}
		fout.close();
		return 1;
	}
}

/* reads dot locations to file ""Quadrant%i.txt".*/
int ReadDotLocations( target *calib_pattern, int quadrant_i){
	char namestring[50];
	sprintf(namestring,"Quadrant%i.txt",quadrant_i);
	ifstream fin(namestring);
	if (fin.fail()){
		cout<<"Failed to read "<<namestring<<".  Press Enter to continue..."<<endl;
		cin.get();
		return 0;
	}
	else{
		cout<<endl<<"Reading dot locations from "<<namestring<<endl<<endl;
		for (int i=0; i<9; i++){
			fin>>calib_pattern->dots[i].ix;
			fin>>calib_pattern->dots[i].iy;
			cout<<setw(10)<<calib_pattern->dots[i].ix<<setw(10)<<calib_pattern->dots[i].iy<<endl;
		}
		calib_pattern->numDots = 9;
		fin.close();
		return 1;
	}
}

/* writes dot spacing from file "calibration_dot_spacing.txt".*/
int WriteDotSpacing(double horizontal_spacing, double vertical_spacing){
	char namestring[50];
	sprintf(namestring,"calibration_dot_spacing.txt");
	ofstream fout(namestring,ios_base::trunc);
	if (fout.fail()){
		cout<<"Failed to write "<<namestring<<".  Press Enter to continue..."<<endl;
		cin.get();
		return 0;
	}
	else{
		cout<<endl<<"Writing dot locations to "<<namestring<<endl<<endl;
		fout<<horizontal_spacing<<" "<<vertical_spacing<<endl;
		fout.close();
		return 1;
	}
}


/* reads dot spacing from file "calibration_dot_spacing.txt".*/
int ReadDotSpacing(double *horizontal_spacing, double *vertical_spacing){
	char namestring[50];
	sprintf(namestring,"calibration_dot_spacing.txt");
	ifstream fin(namestring);
	if (fin.fail()){
		cout<<"Failed to read "<<namestring<<".  Press Enter to continue..."<<endl;
		cin.get();
		return 0;
	}
	else{
		cout<<endl<<"Reading dot spacing from "<<namestring<<endl<<endl;
		for (int i=0; i<9; i++){
			fin>>*horizontal_spacing;
			fin>>*vertical_spacing;
			cout<<"Horizontal spacing: "<<*horizontal_spacing;
			cout<<"Vertical spacing: "<<*vertical_spacing<<endl;
		}
		fin.close();
		return 1;
	}
}

#endif