#ifndef __XBEEPACKETS_H__
#define __XBEEPACKETS_H__
#include "serialport.h"
#include <iostream>
#include <conio.h>
#include <windows.h>


using namespace std;

#define API_MODE_2

#define NUMBER_OF_AGENTS 10

#define ADDITIONAL_NUMS 8 //originally 5, may want to change this to 8 to account for the 3 new RGB values, but doesn't really matter with vision system b/c it should	only be sending smaller (5) b/c it has no RGB data to send, but it might confuse the xbee's packet reading protocol which has been modified to send (and read) packets with 8 additional nums
#define NUM_DATA_SETS 5
#define NUMBERS_PER_SET 2
#define DATATYPE_BYTELENGTH 4
#define DATA_ARRAY_LENGTH (NUM_DATA_SETS*NUMBERS_PER_SET+ADDITIONAL_NUMS)

#define LOAD_TEMP_BYTE 		if ( !getByte(tempByte) ){return 0;}

//This union is used to split a 32-bit float into 4 bytes	
volatile union packetFloat{
	float dataFloat;
	unsigned char dataChar[4];
};

typedef struct XBeeInPacket{
	unsigned int length;
	unsigned char API_ID;
	unsigned int source;
	unsigned char RSSI;
	unsigned char options;
	union packetFloat{
		float dataFloat;
		unsigned char dataChar[4];
	} data[DATA_ARRAY_LENGTH];
	unsigned char checksum;
}XBeeInPacket;

class PacketHandler{
public:
	//Constructor takes serial port to bind to as argument
	PacketHandler (CSerialPort& port){
		pCOM = &port;
		numTotalPackets = 0;
		numDroppedPackets=0;
		numCorruptPackets=0;
		for (int i=0;i<NUMBER_OF_AGENTS;i++){
			receivedStats[i]=0;
		}
	}
	int getNumPackets(void){
		return numTotalPackets;
	}
	int getNumDropped(void){
		return numDroppedPackets;
	}
	int getNumCorrupt(void){
		return numCorruptPackets;
	}
	int getReceivedStats(int i){
		return receivedStats[i];
	}
	void resetStats(void){
		numTotalPackets = 0;
		numDroppedPackets=0;
		numCorruptPackets=0;
		for (int i=0;i<NUMBER_OF_AGENTS;i++){
			receivedStats[i]=0;
		}
	}
	//get a packet
	int getPacket( XBeeInPacket& inPacket ){
		//return value Error Codes//
		//0: action canceled by user
		//1: successfully read
		//2: start delimiter wrong, aborted by user
		//3: checksum incorrect

		unsigned char tempByte;
		unsigned char verifySum=0;
		//cout<<"getting packet, press 'a' to abort..."<<endl;
		//cout<<"press 'r' to reset stats...<<endl;

		numTotalPackets++;

		LOAD_TEMP_BYTE
		if ( tempByte != 0x7E ){
			cout<<"start delimiter wrong, packet dropped."<<endl;
			cout<<"Waiting for next packet, press 'a' to abort"<<endl;
			numDroppedPackets++;
			while ( tempByte!=0x7E ){
				if (GetKeyState('A')<0){ //if 'a' is pressed, abort reading
					return 2;
				}

				else{
					LOAD_TEMP_BYTE;//read to find next 0x7E
				}
				
			}

		}
		LOAD_TEMP_BYTE
			inPacket.length=tempByte*256;
		LOAD_TEMP_BYTE
			inPacket.length=tempByte+inPacket.length;
		LOAD_TEMP_BYTE
			inPacket.API_ID=tempByte;
		LOAD_TEMP_BYTE
			inPacket.source=tempByte*256;
		LOAD_TEMP_BYTE
			inPacket.source=inPacket.source+tempByte;
		LOAD_TEMP_BYTE
			inPacket.RSSI=tempByte;
		LOAD_TEMP_BYTE
			inPacket.options=tempByte;
		//load bytes into the data union.  4 "chars" make up one "float"
		for (int i=0; i<DATA_ARRAY_LENGTH+ADDITIONAL_NUMS; i++){	// 7/10 added "additional_nums" (wasn't there earlier), works with the supersized (8 as opposed to 5) packets from the RGB e-Puck code
			for (int j=0; j<=3; j++){
				LOAD_TEMP_BYTE
				inPacket.data[i].dataChar[3-j]=tempByte;
#if defined API_MODE_2
				if (inPacket.data[i].dataChar[3-j]==0x7D){
					LOAD_TEMP_BYTE
					inPacket.data[i].dataChar[3-j]=tempByte^0x20;
				}
#endif
				verifySum+=inPacket.data[i].dataChar[3-j];
			}
		}
		LOAD_TEMP_BYTE	
		inPacket.checksum=tempByte;

		verifySum+=inPacket.API_ID 
				+ inPacket.source 
				+ inPacket.RSSI 
				+ inPacket.options 
				+ inPacket.checksum;

		if (verifySum == 0xFF){
			cout<<"Checksum correct!"<<endl;
			receivedStats[inPacket.source]++;
			return 1;
		}
		else{
			cout<<"Checksum incorrect!"<<endl;
			numCorruptPackets++;
			return 3;	
		}
	}

	
	int assemblePacket(float rawData[], UINT16 destination){
		unsigned int i; //i is the index
		unsigned int j;//j is for splitting up FLOAT into BYTEs
		union packetFloat tempFloatUnion;
		
		unsigned int index=0;
		unsigned int *indexPtr;
		indexPtr=&index;
		
	//assemble outPacket
		initPacket(outPacket,indexPtr); 
		loadByte(outPacket,0x01,indexPtr);
		loadByte(outPacket,0x01,indexPtr);
		loadByte(outPacket,destination/256,indexPtr);
		loadByte(outPacket,destination%256,indexPtr);
		loadByte(outPacket,0x00,indexPtr);
		
		for (i=0; i<DATA_ARRAY_LENGTH; i++){
			tempFloatUnion.dataFloat=rawData[i];
			for (j=0; j<=3; j++){
				loadByte(outPacket,tempFloatUnion.dataChar[3-j],indexPtr); //read in byte
			}
		}
		loadByte(outPacket,0xFF-outChecksum,indexPtr);
		int bytes_written;
	    bytes_written=sendPacket(outPacket,index);

		return bytes_written;
	}
private:
	//variables
	CSerialPort* pCOM;
	COMSTAT comstat;
	int receivedStats[NUMBER_OF_AGENTS];
	int numDroppedPackets; //Encountered 0x7E before the end of packet
	int numCorruptPackets; //Checksum is wrong
	int numTotalPackets;
	unsigned char outChecksum;
	unsigned char outPacket[90];
	//functions+
	bool getByte( unsigned char & charBuffer ){
		//cout<<"getting byte..."<<endl;
		bool messageFlag=0;
		pCOM->GetStatus(comstat);
		while( comstat.cbInQue==0 ){ 
			if ( GetKeyState('A') < 0 ){
				return 0;
			}
			else if ( GetKeyState('R')<0){
			resetStats();
			}
			pCOM->GetStatus(comstat);
		}
		pCOM->Read( &charBuffer,1 );
		if (charBuffer==0x7D){
			while( comstat.cbInQue==0 ){ 
				if ( GetKeyState('A') < 0 ){
					return 0;
				}
				else if ( GetKeyState('R')<0){
				resetStats();
				}
				pCOM->GetStatus(comstat);
			}
			pCOM->Read( &charBuffer,1 );
			charBuffer=charBuffer^0x20;
		}
		return 1;
	}
	
	//For sending:
	void initPacket(unsigned char outPacket[], unsigned int *pIndex){
		*pIndex=0;
		outChecksum=0;	
		outPacket[*pIndex]=0x7E;
		*pIndex+=1;
		outPacket[*pIndex]=((DATA_ARRAY_LENGTH*DATATYPE_BYTELENGTH+5)/256);
#if defined API_MODE_2
		if (outPacket[*pIndex] == 0x7E ||
			outPacket[*pIndex] == 0x7D ||
			outPacket[*pIndex] == 0x11 ||
			outPacket[*pIndex] == 0x13){	
			outPacket[*pIndex+1] = outPacket[*pIndex]^0x20;
			outPacket[*pIndex] = 0x7D;
			*pIndex+=1;
		}
#endif
		*pIndex+=1;
		outPacket[*pIndex]=((DATA_ARRAY_LENGTH*DATATYPE_BYTELENGTH+5)%256);
#if defined API_MODE_2
		if (outPacket[*pIndex] == 0x7E ||
			outPacket[*pIndex] == 0x7D ||
			outPacket[*pIndex] == 0x11 ||
			outPacket[*pIndex] == 0x13){	
			outPacket[*pIndex+1] = outPacket[*pIndex]^0x20;
			outPacket[*pIndex] = 0x7D;
			*pIndex+=1;
		}
#endif
		*pIndex+=1;
	}

	void loadByte(unsigned char outPacket[],unsigned char byte, unsigned int *pIndex){
		outPacket[*pIndex]=byte;
		outChecksum+=outPacket[*pIndex];
#if defined API_MODE_2
		if (outPacket[*pIndex] == 0x7E ||
			outPacket[*pIndex] == 0x7D ||
			outPacket[*pIndex] == 0x11 ||
			outPacket[*pIndex] == 0x13){	
			outPacket[*pIndex+1] = outPacket[*pIndex]^0x20;
			outPacket[*pIndex] = 0x7D;
			*pIndex+=1;
		}
#endif
		*pIndex+=1;
	}
	int sendPacket(unsigned char outPacket[],unsigned int index){
		//index is the index of the char array not the length
		int written;
		written=pCOM->Write(outPacket,index);
		return written;
	}
};

#endif